from typing import List
from sanic import Sanic
from sanic.views import HTTPMethodView
# from sanic_cors import CORS
from .di.DiContainer import DiContainer
from .config.Environment import Environment
from .config.ConfigProvider import ConfigProvider
from .router.Router import Router
from .database.OrmSessionFactory import OrmSessionFactory
from .view.JinjaRenderer import JinjaRenderer
from .logs.SentryLogger import SentryLogger
from .caching.GlobalCacheRepository import GlobalCacheRepository
from .session.Session import Session
from .database.redis.Redis import Redis


class Insanic:

    def __init__(
        self,
        env: Environment = Environment.PRODUCTION,
        config_file: str = None,
        config_files: List[str] = None
    ):
        container = DiContainer()
        container.init(ConfigProvider, env=env, file=config_file, files=config_files)
        container.init(JinjaRenderer)
        self._container = container


    def route(self, view: HTTPMethodView, route: str):
        router = self._container.get(Router)
        router.route(view, route)


    def static(self, url_path: str, directory: str):
        app = self._container.get(Sanic)
        app.static(url_path, directory)


    def run(self):
        container = self._container
        config = container.get(ConfigProvider)

        if config.has_section(SentryLogger.CONFIG_KEY):
            container.init(SentryLogger)

        if config.has_section(OrmSessionFactory.CONFIG_KEY):
            container.init(OrmSessionFactory)

        if config.has_section(Redis.CONFIG_KEY):
            container.init(Redis)

        container.init(Router)
        container.init(GlobalCacheRepository)
        container.init(Session)

        app = container.get(Sanic)

        is_debug = not config.is_production()
        # if is_debug:
        #     CORS(app, automatic_options=True)

        app.run(host='0.0.0.0', port=5000, workers=1, debug=is_debug, auto_reload=False)
