from typing import List, Optional
import yaml
from pathlib import Path
from .Environment import Environment


class ConfigProvider:

    def __init__(self, env: Environment = Environment.PRODUCTION, file: str = None, files: List[str] = None):
        self.env = env

        if files is None:
            files = []

        if file is not None:
            files.append(file)

        self._config = {}
        for file in files:
            config = yaml.full_load(Path(file).read_text())
            self._config = {**self._config, **config}


    def get_section(self, key) -> Optional[dict]:
        return self._config.get(key)


    def has_section(self, key) -> bool:
        return bool(self._config.get(key))


    def is_production(self) -> bool:
        return self.env is Environment.PRODUCTION
