# import aioredis
from aioredis import Redis, create_redis_pool
from ...config.ConfigProvider import ConfigProvider


class Redis:

    CONFIG_KEY = 'redis'


    def __init__(self, config_provider: ConfigProvider):
        self._config = config_provider.get_section(self.CONFIG_KEY)
        self._client = None


    async def set(self, key: str, value):
        client = await self._get_client()

        if isinstance(value, dict):
            return await client.hmset_dict(key, value)
        else:
            return await client.set(key, value)


    async def get_all(self, key_pattern: str) -> dict:
        client = await self._get_client()
        keys = await client.keys(key_pattern, encoding='utf-8')

        result = {}
        for key in keys:
            result[key] = await self._get(key)

        return result


    async def _get(self, key: str):
        client = await self._get_client()
        return await client.get(key, encoding='utf-8')


    async def _hgetall(self, key: str):
        client = await self._get_client()
        return await client.hgetall(key, encoding='utf-8')


    async def _get_client(self) -> Redis:
        if self._client:
            return self._client

        self._client = await create_redis_pool(self._config['address'], db=self._config['db'])
        return self._client
