import googlemaps
from ..config.ConfigProvider import ConfigProvider
from .GoogleMapsLocation import GoogleMapsLocation


class GoogleMapsClient:

    def __init__(self, config: ConfigProvider):
        api_key = config.get_section('googleMaps')['apiKey']
        self._client = googlemaps.Client(key=api_key)


    def geocode(self, address: str) -> GoogleMapsLocation:
        results = self._client.geocode(address)

        # if result['status'] != 'OK':
        #     raise RuntimeError('Geocoding failed: ' + result['status'])

        if len(results) != 1:
            raise RuntimeError('Geocoding failed: ambiguous results')

        result = results[0]

        location = GoogleMapsLocation()
        location.formatted_address = result['formatted_address']

        address_components = result['address_components']
        for component in address_components:
            if 'street_number' in component['types']:
                street = location.street or ''
                location.street = street.strip() + ' ' + component['long_name']
                continue

            if 'route' in component['types']:
                street = location.street or ''
                location.street = component['long_name'] + ' ' + street.strip()
                continue

            if 'street_address' in component['types']:
                location.street = component['long_name']
                continue

            if 'neighborhood' in component['types']:
                location.neighborhood = component['long_name']
                continue

            if 'administrative_area_level_2' in component['types']:
                location.city = component['long_name']
                continue

            if 'country' in component['types']:
                location.country = component['long_name']
                location.country_code = component['short_name']
                continue

            if 'postal_code' in component['types']:
                location.postal_code = component['long_name']
                continue

        geometry = result['geometry']['location']
        location.lat = geometry['lat']
        location.lng = geometry['lng']

        return location
