from typing import List
from ..config.ConfigProvider import ConfigProvider
from ..database.mixins.EntityImage import EntityImage
from ..database.mixins.GallerySubject import GallerySubject
from .ImageDownloader import ImageDownloader


class Gallery:

    def __init__(self, config: ConfigProvider, subject: GallerySubject):
        self._images_dir = config.get_section('gallery')['imagesDir']
        self._subject = subject


    def get_links(self, width: int = None) -> List[str]:
        images = self._subject.get_images()
        return [self.get_link(x) for x in images]


    def get_link(self, image: EntityImage, width: int = None) -> str:
        link = '/' + self._images_dir + '/' + image.filename
        return link


    def get_thumbnail_link(self, width: int = None) -> str:
        image = self._subject.get_images()[0]
        return self.get_link(image, width=width)


    def download_originals(self):
        for image in self._subject.get_images():
            if image.origin_url is not None and image.filename is None:
                if image.id is None:
                    raise RuntimeError('Images with NULL id cannot be properly named')

                image.filename = str(image.id) + '.jpg'
                path = self._images_dir + '/' + image.filename

                ImageDownloader.fetch(image.origin_url, path)
