from typing import Callable
from functools import wraps
from sanic import response
from .ContextualView import ContextualView


class ApiView(ContextualView):

    @staticmethod
    def marshal_with(schema_cls: type, many: bool = None) -> Callable:
        def apply_marshaling(func):
            @wraps(func)
            async def wrapper(*args, **kwargs) -> str:
                resource = await func(*args, **kwargs)
                schema = schema_cls(many=many)
                dump = schema.dump(resource)
                return response.json(dump)

            return wrapper
        return apply_marshaling


    @staticmethod
    def jsonify(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            resource = await func(*args, **kwargs)
            return response.json(resource)

        return wrapper
