from sanic.views import HTTPMethodView
from ..di.DiContainer import DiContainer
from .ContextualViewDecorator import ContextualViewDecorator


class ContextualView(HTTPMethodView):

    @classmethod
    def as_view(cls: type, container: DiContainer):
        """Return view function for use with the routing system, that
        dispatches request to appropriate handler method.
        """

        def view(*args, **kwargs):
            context = container.fork()
            self = context.get(view.view_class)
            try:
                return self.dispatch_request(*args, **kwargs)
            finally:
                context.teardown()


        if cls.decorators:
            view.__module__ = cls.__module__
            for decorator in cls.decorators:
                if issubclass(decorator, ContextualViewDecorator):
                    decorator = container.get(decorator).get_decorator()

                view = decorator(view)


        view.view_class = cls
        view.__doc__ = cls.__doc__
        view.__module__ = cls.__module__
        view.__name__ = cls.__name__
        return view
